/*
								+---------------------------------+
								|                                 |
								|       ***   Load3DS   ***       |
								|                                 |
								|  Copyright   -tHE SWINe- 2003  |
								|                                 |
								|           Load3DS.cpp           |
								|                                 |
								+---------------------------------+
*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "Vector.h"
#include "Load_3ds.h"

enum {
	Error = -1,
	Ok = 0
};

enum {
    Unknown = 0,
    Camera_Misc,
	Camera_Tar,
	Camera_Pos,
	Object_Pos
};

struct ChunkHeader { 
	unsigned short n_id;
	unsigned long n_len;
};
// kad chunk (balek dat, jako Lump u Q3) m id a dlku

static World *world;
static Object *object;
static Camera *camera;
static Material *material;
static Light *light;
//
static int object_number;
static int face_num;
//
static char name[64];
static char track_name[64];
//
static int track_type, camera_id, object_id;
static int material_id, frame_id, element_id;
static int light_id;
//
static int b_countstuff;

static int Chunk_Read(FILE *fp, long n_pos);

// spot rovnici roviny pro vechny facy v objektu
void Compute_Face_Normals(Object *p_object)
{
	Vector3f v_tmp;
	Vertex **v;
	int i;

	for(i = 0; i < p_object->n_face_num; i ++) {
		v = p_object->face[i].vertex;
		//
		p_object->face[i].p_norm = Planef(v[0]->v_pos - v[2]->v_pos,
			v[1]->v_pos - v[2]->v_pos, v[2]->v_pos);
		// normal

		p_object->face[i].v_center = (p_object->face[i].vertex[0]->v_pos +
			p_object->face[i].vertex[1]->v_pos + p_object->face[i].vertex[2]->v_pos) * (1 / 3.f);
		//
		v_tmp = p_object->face[i].v_center - p_object->face[i].vertex[0]->v_pos;
		p_object->face[i].f_radius = v_tmp.Len();
		//
		v_tmp = p_object->face[i].v_center - p_object->face[i].vertex[1]->v_pos;
		p_object->face[i].f_radius = (p_object->face[i].f_radius < v_tmp.Len())?
			v_tmp.Len() : p_object->face[i].f_radius;
		//
		v_tmp = p_object->face[i].v_center - p_object->face[i].vertex[2]->v_pos;
		p_object->face[i].f_radius = (p_object->face[i].f_radius < v_tmp.Len())?
			v_tmp.Len() : p_object->face[i].f_radius;
		// bounding sphere
	}
}

// spot rovnici roviny pro vechny vertexy v objektu
void Compute_Vertex_Normals(Object *p_object)
{
	int i, j, k; 

	for(i = 0; i < p_object->n_vertex_num; i ++){
		p_object->vertex[i].v_norm.x = 0;
		p_object->vertex[i].v_norm.y = 0;
		p_object->vertex[i].v_norm.z = 0;
		// nastav normlu vrcholu na O = (0, 0, 0)

		for(j = 0; j < p_object->n_face_num; j ++) {
			for(k = 0; k < 3; k ++) {
				//if(p_object->face[j].vertex[k] == &p_object->vertex[i]) { // flat
				if(p_object->face[j].vertex[k]->v_pos.x == p_object->vertex[i].v_pos.x &&
				   p_object->face[j].vertex[k]->v_pos.y == p_object->vertex[i].v_pos.y &&
				   p_object->face[j].vertex[k]->v_pos.z == p_object->vertex[i].v_pos.z) { // smooth
					p_object->vertex[i].v_norm.x += p_object->face[j].p_norm.v_normal.x;
					p_object->vertex[i].v_norm.y += p_object->face[j].p_norm.v_normal.y;
					p_object->vertex[i].v_norm.z += p_object->face[j].p_norm.v_normal.z;
				}
			}
		}
		// pit normly fac

		p_object->vertex[i].v_norm.Normalize();
		// vytvo jednotkov vektor
	}
}

static int Set_Object_Id(int n_index, int n_id, int n_parent_id)
{
	world->object[n_index].n_id = n_id;
	world->object[n_index].n_parent = n_parent_id;
	// nastav id a parent id objektu
	// (parent je pro hierarchick animace)

	return Ok;
}

static int Get_Object(char *name)
{
	int i;

	for(i = 0; i < world->n_object_num; i ++) {
		if(!strcmp(world->object[i].name, name))
			return i;
	} // hled objekt podle jmna
	
	return Error;
}

static int Get_Camera(char *name)
{
	int i;

	for(i = 0; i < world->n_camera_num; i ++) {
		if(!strcmp(world->camera[i].name, name))
			return i;
	} // hled kameru podle jmna
	
	return Error;
}

static int Get_Light(char *name)
{
	int i;

	for(i = 0; i < world->n_light_num; i ++) {
		if(!strcmp(world->light[i].name, name))
			return i;
	} // hled svtlo podle jmna
	
	return Error;
}

static int Get_Material(char *name)
{
	int i;

	for(i = 0; i < world->n_material_num; i ++) {
		if(!strcmp(world->material[i].name, name))
			return i;
	} // hled materil (texturu) podle jmna
	
	return Error;
}

enum {
	CHUNK_VERSION           = 0x0002,
	CHUNK_COLOR_F           = 0x0010,
	CHUNK_COLOR_24          = 0x0011,
	CHUNK_LIN_COLOR24       = 0x0012,
	CHUNK_LIN_COLOR_F       = 0x0013,
	CHUNK_N_PERCENTAGE      = 0x0030,
	CHUNK_F_PERCENTAGE      = 0x0031,
	CHUNK_MASTER_SCALE      = 0x0100,
	CHUNK_BITMAP            = 0x1100,
	CHUNK_USE_BITMAP        = 0x1101,
	CHUNK_BKGND_COLOR       = 0x1200,
	CHUNK_AMBIENT_COLOR     = 0x2100,
	CHUNK_LAYER_FOG         = 0x2302,
	CHUNK_MESHDATA          = 0x3D3D,
	CHUNK_MLIB_MAGIC        = 0x3DAA,
	CHUNK_MESH_VERSION      = 0x3D3E,
	CHUNK_NAMED_OBJECT      = 0x4000,
	CHUNK_TRI_OBJECT        = 0x4100,
	CHUNK_POINT_LIST        = 0x4110,
	CHUNK_POINT_FLAGS       = 0x4111,
	CHUNK_FACE_LIST         = 0x4120,
	CHUNK_MATERIAL_GROUP    = 0x4130,
	CHUNK_TEX_VERTS         = 0x4140,
	CHUNK_SMOOTH_GROUP      = 0x4150,
	CHUNK_MESH_MATRIX       = 0x4160,
	CHUNK_MESH_COLOR        = 0x4165,
	CHUNK_TEXTURE_INFO      = 0x4170,
	CHUNK_MESH_BOXMAP       = 0x4190,
	CHUNK_LIGHT             = 0x4600,
	CHUNK_SPOTLIGHT         = 0x4610,
	CHUNK_LIGHT_OFF         = 0x4620,
	CHUNK_LIGHT_SHADOWED    = 0x4627,
	CHUNK_LIGHT_ATTENT      = 0x4625,
	CHUNK_LIGHT_RANGE       = 0x4655,
	CHUNK_LIGHT_RAY_BIAS    = 0x4658,
	CHUNK_LIGHT_IN_RANGE    = 0x4659,
	CHUNK_LIGHT_OUT_RANGE   = 0x465A,
	CHUNK_LIGHT_MULTIPLIER  = 0x465B,
	CHUNK_CAMERA            = 0x4700,
	CHUNK_MAIN_MAGIC        = 0x4D4D,
	CHUNK_HIERARCHY         = 0x4F00,
	CHUNK_VIEWPORT_LAYOUT   = 0x7001,
	CHUNK_VIEWPORT_SIZE     = 0x7020,
	CHUNK_MATERIAL          = 0xAFFF,
	CHUNK_MATERIAL_NAME     = 0xA000,
	CHUNK_MATERIAL_AMBIENT  = 0xA010,
	CHUNK_MATERIAL_DIFFUSE  = 0xA020,
	CHUNK_MATERIAL_SPECULAR = 0xA030,
	CHUNK_MATERIAL_SHININESS    = 0xA040,
	CHUNK_MATERIAL_SHINSTRENGTH = 0xA041,
	CHUNK_MATERIAL_TRANSPARENCY = 0xA050,
	CHUNK_MATERIAL_XPFALL   = 0xA052,
	CHUNK_MATERIAL_REFBLUR  = 0xA053,
	CHUNK_MATERIAL_SELFILLUM    = 0xA084,
	CHUNK_MATERIAL_DBLSIDED = 0xA081,
	CHUNK_MATERIAL_TRANSADD = 0xA083,
	CHUNK_MATERIAL_WIREON   = 0xA085,
	CHUNK_MATERIAL_WIRESIZE = 0xA087,
	CHUNK_MATERIAL_SOFTEN   = 0xA08C,
	CHUNK_MATERIAL_SHADING  = 0xA100,
	CHUNK_MATERIAL_TEXTURE  = 0xA200,
	CHUNK_MATERIAL_REFLECT  = 0xA220,
	CHUNK_MATERIAL_BUMPMAP  = 0xA230,
	CHUNK_MATERIAL_BUMPAMOUNT   = 0xA252,
	CHUNK_MATERIAL_MAPNAME  = 0xA300,
	CHUNK_MATERIAL_MAPFLAGS = 0xA351,
	CHUNK_MATERIAL_MAPBLUR  = 0xA353,
	CHUNK_MATERIAL_MAPUSCALE    = 0xA354,
	CHUNK_MATERIAL_MAPVSCALE    = 0xA356,
	CHUNK_MATERIAL_MAPUOFFSET   = 0xA358,
	CHUNK_MATERIAL_MAPVOFFSET   = 0xA35A,
	CHUNK_MATERIAL_MAPROTANGLE  = 0xA35C,
	CHUNK_KF_DATA           = 0xB000,
	CHUNK_AMBIENT_KEY       = 0xB001,
	CHUNK_TRACK_INFO        = 0xB002,
	CHUNK_TRACK_CAMERA      = 0xB003,
	CHUNK_TRACK_CAM_TARGET  = 0xB004,
	CHUNK_TRACK_LIGHT       = 0xB005,
	CHUNK_TRACK_LIGHT_TARGET    = 0xB006,
	CHUNK_TRACK_SPOTLIGHT   = 0xB007,
	CHUNK_KF_SEG            = 0xB008,
	CHUNK_KF_CURTIME        = 0xB009,
	CHUNK_KF_HDR            = 0xB00A,
	CHUNK_TRACK_NODE_NAME   = 0xB010,
	CHUNK_INSTANCE_NAME     = 0xB011,
	CHUNK_PRESCALE          = 0xB012,
	CHUNK_TRACK_PIVOT       = 0xB013,
	CHUNK_BOUNDBOX          = 0xB014,
	CHUNK_MORPH_SMOOTH      = 0xB015,
	CHUNK_TRACK_POS         = 0xB020,
	CHUNK_TRACK_ROTATE      = 0xB021,
	CHUNK_TRACK_SCALE       = 0xB022,
	CHUNK_TRACK_FOV         = 0xB023,
	CHUNK_TRACK_ROLL        = 0xB024,
	CHUNK_TRACK_COLOR       = 0xB025,
	CHUNK_TRACK_MORPH       = 0xB026,
	CHUNK_TRACK_HOTSPOT     = 0xB027,
	CHUNK_TRACK_FALLOFF     = 0xB028,
	CHUNK_TRACK_HIDE        = 0xB029,
	CHUNK_NODE_ID           = 0xB030,
	CHUNK_C_MAGIC           = 0xC23D
};
// znaky jednotlivch chunk

static inline int Read_CStr(char *name, FILE *fp)
{
	int i, c;

	i = 0;
	while((c = fgetc(fp)) != EOF && c && i < 64 - 1) 
		name[i ++] = c;
	name[i] = 0;
	
	if(i == 63)
		return Ok;

	return ((c == EOF) ? Error : Ok);
}

static inline int Read_Skip(FILE *fp, long n_pos)
{
	return Ok;
}

static inline int Skip_Float(FILE *fp, long n_pos)
{
	float b[1];

	fread(b, sizeof(b), 1, fp);

	return Ok;
}

static inline int Skip_Word(FILE *fp, long n_pos)
{
	short b[1];

	fread(b, sizeof(b), 1, fp);

	return Ok;
}

static inline int Skip_DWord(FILE *fp, long n_pos)
{
	float b[1];

	fread(b, sizeof(b), 1, fp);

	return Ok;
}

static inline int Read_Color_RGB(FILE *fp, long n_pos)
{
	unsigned char c[3];

	fread(c, sizeof(c), 1, fp);

	return Ok;
}

static inline int Read_Color_RGBF(FILE *fp, long n_pos)
{
	int n_index;
	float c[3];

	fread(c, sizeof(c), 1, fp);

	if(!b_countstuff) {
		if((n_index = Get_Light(name)) != Error) {
			world->light[n_index].v_color.x = c[0];
			world->light[n_index].v_color.y = c[2];
			world->light[n_index].v_color.z = c[1];
			//
			world->light[n_index].r = (int)(c[0] * 255);
			world->light[n_index].g = (int)(c[1] * 255);
			world->light[n_index].b = (int)(c[2] * 255);
		}
	}

	return Ok;
}

static inline int Read_Light(FILE *fp, long n_pos)
{
	float c[3];

	fread(c, sizeof(c), 1, fp);
	if(b_countstuff)
		world->n_light_num ++;
	else {
		light = &world->light[light_id ++];
		//
		if((light->name = (char*)malloc(strlen(name) + 1)) == NULL)
			return Error;
		strcpy(light->name, name);
		//
		light->v_position.x = c[0];
		light->v_position.z = c[1];
		light->v_position.y = c[2];
		//
		light->b_spot = 0;
	}

	return Ok;
}

static inline int Read_SpotLight(FILE *fp, long n_pos)
{
	float c[5];

	fread(c, sizeof(c), 1, fp);
	if(!b_countstuff) {
		light->b_spot = 1;
		//
		light->v_target.x = c[0];
		light->v_target.z = c[1];
		light->v_target.y = c[2];
		//
		light->f_hotspot = c[3];
		light->f_faloff = c[4];
	}

	return Ok;
}

static inline int Read_LightInnerRange(FILE *fp, long n_pos)
{
	float f;

	fread(&f, sizeof(float), 1, fp);
	if(!b_countstuff)
		light->f_inner_range = f;

	return Ok;
}

static inline int Read_LightOuterRange(FILE *fp, long n_pos)
{
	float f;

	fread(&f, sizeof(float), 1, fp);
	if(!b_countstuff)
		light->f_outer_range = f;

	return Ok;
}

static inline int Read_LightMultiplier(FILE *fp, long n_pos)
{
	float f;

	fread(&f, sizeof(float), 1, fp);
	if(!b_countstuff)
		light->f_multiplier = f;

	return Ok;
}

static inline int Read_LightOff(FILE *fp, long n_pos)
{
	if(!b_countstuff)
		light->b_off = 1;

	return Ok;
}

static inline int Read_LightShadowed(FILE *fp, long n_pos)
{
	if(!b_countstuff)
		light->b_cast_shadows = 1;

	return Ok;
}

static inline int Read_MeshName(FILE *fp, long n_pos)
{
	Read_CStr(name, fp);

	return Ok;
}

static inline int Read_MapFile(FILE *fp, long n_pos)
{
	char tmp[64];

	Read_CStr(tmp, fp);

	return Ok;
}

static int Read_NodeId(FILE *fp, long n_pos)
{
	unsigned short num;

	fread(&num, sizeof(short), 1, fp);
	object_number = num;

	return Ok;
}

static int Read_TriObject(FILE *fp, long n_pos)
{
	if(b_countstuff)
		world->n_object_num ++;
	else {
		object = &world->object[object_id ++];
		if((object->name = (char*)malloc(strlen(name) + 1)) == NULL)
			return Error;
		strcpy(object->name, name);
	}

	return Ok;
}

static int Read_VertexList(FILE *fp, long n_pos)
{
	unsigned short n;
	float c[3];
	int i;
	
	fread(&n, sizeof(short), 1, fp);

	if(!b_countstuff) {
		object->n_vertex_num = n;
		if((object->vertex = (Vertex*)malloc(n * sizeof(Vertex))) == NULL)
			return Error;
	}

	i = 0;
	while(i < n) {
		fread(c, sizeof(c), 1, fp);
		if(!b_countstuff) {
			object->vertex[i].v_pos.x = c[0];
			object->vertex[i].v_pos.y = c[2]; // 3D - studio m prohozen osy
			object->vertex[i].v_pos.z = c[1];
		} 
		i ++;
	}

	return Ok;
}

static int Read_VertexFlags(FILE *fp, long n_pos)
{
	unsigned short n, n_flag; 
	
	fread(&n, sizeof(short), 1, fp);
	while(n -- > 0)
		fread(&n_flag, sizeof(short), 1, fp);

	return Ok;
}

static int Read_FaceList(FILE *fp, long n_pos)
{
	unsigned short n, c[3], flags;
	int i;

	fread(&n, sizeof(short), 1, fp);
	face_num = n;
	if(!b_countstuff) {
		object->n_face_num = n;
		if((object->face = (Face*)malloc(n * sizeof(Face))) == NULL)
			return Error;
	}

	i = 0;
	while(i < n) { 
		fread(c, sizeof(c), 1, fp);
		fread(&flags, sizeof(short), 1, fp);
		if(!b_countstuff) {
			object->face[i].vertex[0] = &object->vertex[c[0]];
			object->face[i].vertex[1] = &object->vertex[c[2]];
			object->face[i].vertex[2] = &object->vertex[c[1]];
		} 
		i ++;
	}

	return Ok;
}

static int Read_FaceMaterial(FILE *fp, long n_pos)
{
	char s_material_name[64];
	unsigned short n, n_face; 
	int m, i;
	
	if(Read_CStr(s_material_name, fp) == Error)
		return Error;
	
	if(!b_countstuff)
		m = Get_Material(s_material_name);

	fread(&n, sizeof(short), 1, fp);
	i = 0;
	while(i < n) {
		fread(&n_face, sizeof(short), 1, fp);
		if(!b_countstuff)
			object->face[n_face].n_material = m;
		i ++; 
	}

	return Ok;
}

static int Read_MapList(FILE *fp, long n_pos)
{
	unsigned short n; 
	float c[2]; 
	int i; 
	
	fread(&n, sizeof(short), 1, fp);
	i = 0;
	while(i < n) {
		fread(c, sizeof(c), 1, fp);
		if(!b_countstuff) {
			object->vertex[i].v_tex.x = c[0];
			object->vertex[i].v_tex.y = c[1];
		} 
		i ++;
	}

	return Ok;
}

static int Read_SmoothingList(FILE *fp, long n_pos)
{
	unsigned short n_face_num;
	unsigned long s;

	n_face_num = face_num;
	while(n_face_num -- > 0)
		fread(&s, sizeof(short), 1, fp);

	return Ok;
}

static int Read_MeshMatrix(FILE *fp, long n_pos)
{
	float f_matrix[4][3];

	fread(f_matrix, sizeof(f_matrix), 1, fp);
	if(!b_countstuff) {
		object->m_object.f[0][0] = f_matrix[0][0];
		object->m_object.f[1][0] = f_matrix[1][0];
		object->m_object.f[2][0] = f_matrix[2][0];
		object->m_object.f[3][0] = f_matrix[3][0];
		//
		object->m_object.f[0][1] = f_matrix[0][1];
		object->m_object.f[1][1] = f_matrix[1][1];
		object->m_object.f[2][1] = f_matrix[2][1];
		object->m_object.f[3][1] = f_matrix[3][1];
		//
		object->m_object.f[0][2] = f_matrix[0][2];
		object->m_object.f[1][2] = f_matrix[1][2];
		object->m_object.f[2][2] = f_matrix[2][2];
		object->m_object.f[3][2] = f_matrix[3][2];
		//
		object->m_object.f[0][3] = 0;
		object->m_object.f[1][3] = 0;
		object->m_object.f[2][3] = 0;
		object->m_object.f[3][3] = 1;
	}

	return Ok;
}

static int Read_Camera(FILE *fp, long n_pos)
{
	float c[8];

	fread(c, sizeof(c), 1, fp);

	if(b_countstuff) 
		world->n_camera_num ++; 
	else {
		camera = &world->camera[camera_id ++];
		//
		camera->f_pos_x = c[0];
		camera->f_pos_y = c[1];
		camera->f_pos_z = c[2];
		camera->f_tar_x = c[3];
		camera->f_tar_y = c[4];
		camera->f_tar_z = c[5];
		camera->f_bank_angle = c[6];
		camera->f_focus = c[7];
		//
		if((camera->name = (char*)malloc(strlen(name) + 1)) == NULL)
			return Error;
		strcpy(camera->name, name);
	}

	return Ok;
}

static int Read_MaterialName(FILE *fp, long n_pos)
{
	if(Read_CStr(name, fp) == Error)
		return Error;

	if(b_countstuff) {
		world->n_material_num ++;
	} else {
		material = &world->material[material_id ++];
		if((material->name = (char*)malloc(strlen(name) + 1)) == NULL)
			return Error;
		strcpy(material->name, name);
	}

	return Ok;
}

static int Read_Frames(FILE *fp, long n_pos)
{
	unsigned long c[2]; 

	fread(c, sizeof(c), 1, fp);

	world->n_start_frame = c[0];
	world->n_end_frame = c[1];

	return Ok;
}

static int Read_TrackNodeName(FILE *fp, long n_pos)
{
	unsigned short parent;
	unsigned short w[2];

	if(Read_CStr(track_name, fp) == Error)
		return Error;

	fread(w, sizeof(w), 1, fp);
	fread(&parent, sizeof(short), 1, fp);

	if(b_countstuff) 
		return Ok;
	if(track_type != Unknown) 
		element_id = Get_Camera(track_name);
	else { 
		element_id = Get_Object(track_name);
		Set_Object_Id(element_id, object_number, parent);
		track_type = Object_Pos;
	}

	return Ok;
}

static int Read_PivotPoint(FILE *fp, long n_pos) 
{
	Skip_Float(fp, n_pos);
	Skip_Float(fp, n_pos);
	Skip_Float(fp, n_pos);

	return Ok; 
}

static int Read_SplineFlags(FILE *fp, unsigned short *nf)
{
	unsigned short unknown, flags, nff;
	float f_tension, f_cont, f_bias; 
	unsigned int i;

	fread(&nff, sizeof(short), 1, fp);
	fread(&unknown, sizeof(short), 1, fp);
	fread(&flags, sizeof(short), 1, fp);

	*nf = nff;
	for(i = 0;i < 16;i ++) {
		if(flags & (1 << i))
			switch(i) {
			case 0:
				fread(&f_tension, sizeof(float), 1, fp);
				break;
			case 1: 
				fread(&f_cont, sizeof(float), 1, fp);
				break;
			case 2: 
				fread(&f_bias, sizeof(float), 1, fp);
				break;
			}
	}

	return Ok;
}

static int Read_TrackFlags(FILE *fp, unsigned short *n)
{
	unsigned short flags[7];

	fread(flags, sizeof(flags), 1, fp);
	*n = flags[5];

	return Ok;
}

static int Read_TrackPos(FILE *fp, long n_pos)
{
	unsigned short n, n_flags;
	float pos[3];

	if(Read_TrackFlags(fp, &n) != Ok)
		return Error;

	if(!b_countstuff && element_id >= 0) {
		switch(track_type) {
		case Object_Pos:
			world->object[element_id].n_pos_num = n;
			if((world->object[element_id].pt = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			if((world->object[element_id].px = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			if((world->object[element_id].py = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			if((world->object[element_id].pz = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			frame_id = 0;
			break;

		case Camera_Tar:
			world->camera[element_id].n_target_num = n;
			if((world->camera[element_id].tt = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			if((world->camera[element_id].tx = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			if((world->camera[element_id].ty = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			if((world->camera[element_id].tz = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			frame_id = 0;
			break;

		case Camera_Pos:
			world->camera[element_id].n_position_num = n;
			if((world->camera[element_id].pt = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			if((world->camera[element_id].px = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			if((world->camera[element_id].py = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			if((world->camera[element_id].pz = (float*)malloc(n * sizeof(float))) == NULL)
				return Error;
			frame_id = 0;
			break;
		}
	}

	while(n -- > 0) {
		if(Read_SplineFlags(fp, &n_flags) != Ok) 
			return Error;

		fread(pos, sizeof(pos), 1, fp);

		if(!b_countstuff && element_id >= 0)
			switch(track_type) {
			case Camera_Pos:
				world->camera[element_id].pt[frame_id] = n_flags;
				world->camera[element_id].px[frame_id] = pos[0];
				world->camera[element_id].py[frame_id] = pos[2];
				world->camera[element_id].pz[frame_id] = pos[1];
				break;

			case Camera_Tar:
				world->camera[element_id].tt[frame_id] = n_flags;
				world->camera[element_id].tx[frame_id] = pos[0];
				world->camera[element_id].ty[frame_id] = pos[2];
				world->camera[element_id].tz[frame_id] = pos[1];
				break;

			case Object_Pos:
				world->object[element_id].pt[frame_id] = n_flags;
				world->object[element_id].px[frame_id] = pos[0];
				world->object[element_id].py[frame_id] = pos[2];
				world->object[element_id].pz[frame_id] = pos[1];
				break;
			}

		frame_id ++;
	}

	return Ok;
}

static int Read_TrackColor(FILE *fp, long n_pos)
{
	unsigned short num, n_flags;
	float pos[3];

	if(Read_TrackFlags(fp, &num) != Ok)
		return Error;

	while(num -- > 0) {
		if(Read_SplineFlags(fp, &n_flags)!=Ok) 
			return Error; 
		fread(pos, sizeof(pos), 1, fp);
	}

	return Ok;
}

static int Read_TrackRot(FILE *fp, long n_pos)
{
	unsigned short num, n_flag;
	float pos[4];

	if(Read_TrackFlags(fp, &num) != Ok) 
		return Error;

	if(!b_countstuff && element_id >= 0 && track_type == Object_Pos) {
		world->object[element_id].n_rot_num = num;
		if((world->object[element_id].qt = (float*)malloc(num * sizeof(float))) == NULL)
			return Error;
		if((world->object[element_id].qw = (float*)malloc(num * sizeof(float))) == NULL)
			return Error;
		if((world->object[element_id].qx = (float*)malloc(num * sizeof(float))) == NULL)
			return Error;
		if((world->object[element_id].qy = (float*)malloc(num * sizeof(float))) == NULL)
			return Error;
		if((world->object[element_id].qz = (float*)malloc(num * sizeof(float))) == NULL)
			return Error;
		frame_id = 0;
	}

	while(num -- > 0) {
		if(Read_SplineFlags(fp, &n_flag) != Ok) 
			return Error;

		fread(pos, sizeof(pos), 1, fp);

		if(!b_countstuff && element_id >= 0 && track_type == Object_Pos) {
			world->object[element_id].qt[frame_id] = n_flag;
			world->object[element_id].qw[frame_id] = pos[0];
			world->object[element_id].qx[frame_id] = pos[1];
			world->object[element_id].qy[frame_id] = pos[3];
			world->object[element_id].qz[frame_id] = pos[2];
			frame_id++;
		}
	}

	return Ok;
}

static int Read_TrackFOV(FILE *fp, long n_pos)
{
	unsigned short num, n_flag;
	float pos;

	if(Read_TrackFlags(fp, &num) != Ok)
		return Error;

	while(num -- > 0) {
		if(Read_SplineFlags(fp, &n_flag) != Ok)
			return Error; 
		fread(&pos, sizeof(float), 1, fp);
	}

	return Ok;
}

static int Read_TrackRoll(FILE *fp, long n_pos)
{
	unsigned short num, n_flag;
	float pos;

	if(Read_TrackFlags(fp, &num) != Ok)
		return Error;

	while(num -- > 0) {
		if(Read_SplineFlags(fp, &n_flag) != Ok)
			return Error; 
		fread(&pos, sizeof(float), 1, fp);
	}

	return Ok;
}

static int Read_TrackScale(FILE *fp, long n_pos)
{
	unsigned short num, n_flag;
	float pos;

	if(Read_TrackFlags(fp, &num) != Ok)
		return Error;

	while(num -- > 0) {
		if(Read_SplineFlags(fp, &n_flag) != Ok)
			return Error; 
		fread(&pos, sizeof(pos), 1, fp);
	}

	return Ok;
}

static int Read_TrackCamera(FILE *fp, long n_pos)
{
	track_type = Camera_Pos;

	return Chunk_Read(fp, n_pos);
}

static int Read_TrackCamTgt(FILE *fp, long n_pos)
{
	track_type = Camera_Tar;

	return Chunk_Read(fp, n_pos);
}

static int Read_TrackInfo(FILE *fp, long n_pos)
{
	track_type = Unknown;

	return Chunk_Read(fp, n_pos);
}

struct {
	unsigned short n_id;
	int b_subchunk;
	int (*function)(FILE *fp, long n_pos);
} ChunkNames[] = {
  {CHUNK_VERSION,0, Skip_Word},
  {CHUNK_COLOR_F, 0, Read_Color_RGBF},
  {CHUNK_COLOR_24, 0, Read_Color_RGB},
  {CHUNK_LIN_COLOR24, 0, Read_Color_RGB},
  {CHUNK_LIN_COLOR_F, 0, Read_Color_RGBF},
  {CHUNK_N_PERCENTAGE, 0, Skip_Word},
  {CHUNK_F_PERCENTAGE, 0, Skip_Float},
  {CHUNK_MASTER_SCALE, 0, Skip_Float},
  {CHUNK_C_MAGIC, 1, NULL},
  {CHUNK_MLIB_MAGIC, 1, NULL},
  {CHUNK_MESH_VERSION, 0, Skip_Word},
  {CHUNK_MAIN_MAGIC, 1, NULL},
  {CHUNK_MESHDATA, 1, NULL},
  {CHUNK_BKGND_COLOR, 1, NULL},
  {CHUNK_AMBIENT_COLOR, 1, NULL},
  {CHUNK_NAMED_OBJECT, 1, Read_MeshName},
  {CHUNK_TRI_OBJECT, 1, Read_TriObject},
  {CHUNK_POINT_LIST, 0, Read_VertexList},
  {CHUNK_POINT_FLAGS, 0, Read_VertexFlags},
  {CHUNK_FACE_LIST, 1, Read_FaceList},
  {CHUNK_MATERIAL_GROUP, 0, Read_FaceMaterial},
  {CHUNK_TEX_VERTS, 0, Read_MapList},
  {CHUNK_SMOOTH_GROUP, 0, Read_SmoothingList},
  {CHUNK_MESH_MATRIX, 0, Read_MeshMatrix},
  {CHUNK_MESH_COLOR, 0, Read_Skip},
  {CHUNK_TEXTURE_INFO, 0, Read_Skip},
  {CHUNK_MESH_BOXMAP, 0, NULL},
  {CHUNK_LIGHT, 1, Read_Light},
  {CHUNK_SPOTLIGHT, 0, Read_SpotLight},
  {CHUNK_CAMERA, 0, Read_Camera},
  {CHUNK_HIERARCHY, 1, NULL},
  {CHUNK_VIEWPORT_LAYOUT, 0, Read_Skip},
  {CHUNK_VIEWPORT_SIZE, 0, Read_Skip},
  {CHUNK_MATERIAL, 1, NULL},
  {CHUNK_MATERIAL_NAME, 0, Read_MaterialName},
  {CHUNK_MATERIAL_AMBIENT, 1, NULL},
  {CHUNK_MATERIAL_DIFFUSE, 1, NULL},
  {CHUNK_MATERIAL_SPECULAR, 1, NULL},
  {CHUNK_MATERIAL_SHININESS, 1, NULL},
  {CHUNK_MATERIAL_SHINSTRENGTH, 1, NULL},
  {CHUNK_MATERIAL_TRANSPARENCY, 1, NULL},
  {CHUNK_MATERIAL_XPFALL, 1, NULL},
  {CHUNK_MATERIAL_REFBLUR, 1, NULL},
  {CHUNK_MATERIAL_SELFILLUM, 1, NULL},
  {CHUNK_MATERIAL_DBLSIDED, 0, Read_Skip},
  {CHUNK_MATERIAL_TRANSADD, 0, Read_Skip},
  {CHUNK_MATERIAL_WIREON, 0, Read_Skip},
  {CHUNK_MATERIAL_WIRESIZE, 0, Skip_Float},
  {CHUNK_MATERIAL_SOFTEN, 0, Read_Skip},
  {CHUNK_MATERIAL_SHADING, 0, Skip_Word},
  {CHUNK_MATERIAL_TEXTURE, 1, Read_Skip},
  {CHUNK_MATERIAL_REFLECT, 1, Read_Skip},
  {CHUNK_MATERIAL_BUMPMAP, 1, Read_Skip},
  {CHUNK_MATERIAL_BUMPAMOUNT, 0, Skip_Word},
  {CHUNK_MATERIAL_MAPNAME, 0, Read_MapFile},
  {CHUNK_MATERIAL_MAPFLAGS, 0, Skip_Word},
  {CHUNK_MATERIAL_MAPBLUR, 0, Skip_Float},
  {CHUNK_MATERIAL_MAPUSCALE, 0, Skip_Float},
  {CHUNK_MATERIAL_MAPVSCALE, 0, Skip_Float},
  {CHUNK_MATERIAL_MAPUOFFSET, 0, Skip_Float},
  {CHUNK_MATERIAL_MAPVOFFSET, 0, Skip_Float},
  {CHUNK_MATERIAL_MAPROTANGLE, 0, Skip_Float},
  {CHUNK_KF_DATA, 1, NULL},
  {CHUNK_AMBIENT_KEY, 1, NULL},
  {CHUNK_TRACK_INFO, 1, Read_TrackInfo},
  {CHUNK_KF_SEG, 0, Read_Frames},
  {CHUNK_TRACK_NODE_NAME, 0, Read_TrackNodeName},
  {CHUNK_INSTANCE_NAME, 0, NULL},
  {CHUNK_TRACK_PIVOT, 0, Read_PivotPoint},
  {CHUNK_TRACK_POS, 0, Read_TrackPos},
  {CHUNK_TRACK_COLOR, 0, Read_TrackColor},
  {CHUNK_TRACK_ROTATE, 0, Read_TrackRot},
  {CHUNK_TRACK_SCALE, 0, Read_TrackScale},
  {CHUNK_TRACK_MORPH, 0, NULL},
  {CHUNK_TRACK_HIDE, 0, NULL},
  {CHUNK_NODE_ID, 0, Read_NodeId},
  {CHUNK_TRACK_CAMERA, 1, Read_TrackCamera},
  {CHUNK_TRACK_CAM_TARGET, 1, Read_TrackCamTgt},
  {CHUNK_TRACK_LIGHT, 1, NULL},
  {CHUNK_TRACK_LIGHT_TARGET, 1, NULL},
  {CHUNK_TRACK_SPOTLIGHT, 1, NULL},
  {CHUNK_TRACK_FOV, 0, Read_TrackFOV},
  {CHUNK_TRACK_ROLL, 0, Read_TrackRoll},
  {CHUNK_LIGHT_IN_RANGE, 0, Read_LightInnerRange},
  {CHUNK_LIGHT_OUT_RANGE, 0, Read_LightOuterRange},
  {CHUNK_LIGHT_MULTIPLIER, 0, Read_LightMultiplier},
  {CHUNK_LIGHT_OFF, 0, Read_LightOff},
  {CHUNK_LIGHT_SHADOWED, 0, Read_LightShadowed},
};

/*
Read_LightInnerRange(FILE *fp, long n_pos)
static inline int Read_LightOuterRange(FILE *fp, long n_pos)
static inline int Read_LightMultiplier(FILE *fp, long n_pos)
static inline int Read_LightOff(FILE *fp, long n_pos)
static inline int Read_LightShadowed(FILE *fp, long n_pos)
*/

static int Chunk_Find(unsigned short n_id)
{
	int i, chunk_num;

	chunk_num = sizeof(ChunkNames) / sizeof(ChunkNames[0]);
	for(i = 0; i < chunk_num; i ++) {
		if(n_id == ChunkNames[i].n_id) 
			return i;
	}
	// hled chunk podle id

	return Error;
}

static int Chunk_Read(FILE *fp, long n_pos)
{
	int n, n_result;
	long n_cur_pos;
	ChunkHeader h;

	while(ftell(fp) < n_pos) {
		n_cur_pos = ftell(fp);
		fread(&h.n_id, sizeof(short), 1, fp);
		fread(&h.n_len, sizeof(long), 1, fp);
		//
		if(!h.n_len)
			return Error;
		// pete chunk

		n = Chunk_Find(h.n_id);
		if(n < 0)
			fseek(fp, n_cur_pos + h.n_len, SEEK_SET);
		    // zkus ho najt v tabulce, pokud ho nenajde tak ho pesko
		else { 
			n_cur_pos += h.n_len;
			if(ChunkNames[n].function != NULL) {
				if((n_result = ChunkNames[n].function(fp, n_cur_pos)) != Ok)
					return n_result;
			}
			// pokud m funkci pro ten tak se pete

			if(ChunkNames[n].b_subchunk) {
				if((n_result = Chunk_Read(fp, n_cur_pos)) != Ok)
					return n_result;
			}
			// pokud m subchunky, te subchunky
			fseek(fp, n_cur_pos, SEEK_SET);
		}
		if(ferror(fp))
			return Error;
	}

	return Ok;
}

// ------------------------------------

void CreateMatrix(Camera *c_camera)
{
	Vector3f v_direction, v_right, v_up;

	if(camera->px == NULL || camera->py == NULL || camera->pz == NULL) {
		camera->m_camera.Init();
		return;
	}

	if(camera->tx == NULL || camera->tx == NULL || camera->tx == NULL) {
		camera->m_camera.Init();
		//
		camera->m_camera.f[3][0] = camera->px[0];
		camera->m_camera.f[3][1] = camera->pz[0];
		camera->m_camera.f[3][2] = camera->py[0];

		return;
	}

	v_up.x = 0; 
	v_up.y = 1; 
	v_up.z = 0;
	v_direction.x = camera->tx[0] - camera->px[0];
	v_direction.y = camera->ty[0] - camera->py[0];
	v_direction.z = camera->tz[0] - camera->pz[0];

	v_direction.Normalize();
	v_right = v_direction * v_up;
	v_up = v_direction * v_right;
	v_right.Normalize();
	v_up.Normalize();

	camera->m_camera.Init();
	//
	camera->m_camera.f[0][0] = v_right.x;
	camera->m_camera.f[0][1] = v_right.y;
	camera->m_camera.f[0][2] = v_right.z;
	camera->m_camera.f[1][0] = v_up.x;
	camera->m_camera.f[1][1] = v_up.y;
	camera->m_camera.f[1][2] = v_up.z;
	camera->m_camera.f[2][0] = v_direction.x;
	camera->m_camera.f[2][1] = v_direction.y;
	camera->m_camera.f[2][2] = v_direction.z;
	camera->m_camera.f[3][0] = camera->px[0];
	camera->m_camera.f[3][1] = camera->py[0];
	camera->m_camera.f[3][2] = camera->pz[0];
}

int World::Load(const char *file_name)
{
	long f_len;
	FILE *fp;
	int i;

	world = this;

	n_material_num = 0;
	n_camera_num = 0;
	n_object_num = 0;
	n_light_num = 0;
	// naalokuje svt, piprav pro nahrvn

	b_countstuff = 1;
	if((fp = fopen(file_name, "rb")) == NULL)
		return false;
	// oteve soubor

	fseek(fp, 0, SEEK_END);
	f_len = ftell(fp);
	fseek(fp, 0, SEEK_SET);
	// najde konec

	i = Chunk_Read(fp, f_len);
	// te chunky

	fflush(fp);
	fclose(fp);
	if(i != Ok)
		return false;
	// uzave soubor a zkontroluje chyby

	material = new Material[n_material_num];
	object = new Object[n_object_num];
	camera = new Camera[n_camera_num];
	light = new Light[n_light_num];
	// naalokuje pam

	memset(material, 0, sizeof(Material) * n_material_num);
	memset(object, 0, sizeof(Object) * n_object_num);
	memset(camera, 0, sizeof(Camera) * n_camera_num);
	memset(light, 0, sizeof(Light) * n_light_num);
	b_countstuff = 0;
	object_id = 0;
	camera_id = 0;
	material_id = 0;
	light_id = 0;
	// nastav promnn

	if((fp = fopen(file_name, "rb")) == NULL)
		return false;
	// oteve podruh (poprv to bylo jen pro spotn objekt ..)

	i = Chunk_Read(fp, f_len);
	// te se

	fflush(fp);
	fclose(fp);
	if(i != Ok)
		return false;
	// uzave soubor a zkontroluje chyby
	
	for(i = 0; i < n_object_num; i ++) {
		Compute_Face_Normals(&object[i]);
		Compute_Vertex_Normals(&object[i]);
	}
	// spot normly

	for(i = 0; i < n_camera_num; i ++)
		CreateMatrix(&camera[i]);
	// spot kamery

	return true;
}

int Face::b_Contain_Vertex(Vector3f &v_point)
{
	Planef p_plane;
	Vector3f v_vec;
	int i, n_prev;

	n_prev = 2;
	for(i = 0; i < 3; n_prev = i ++) {
		v_vec = vertex[i]->v_pos - vertex[n_prev]->v_pos;
		v_vec.Normalize();
		//
		p_plane = Planef(p_norm.v_normal, v_vec, vertex[i]->v_pos);
		//
		if(p_plane.Vector_Pos(v_point) == Pl_Back)
			return false;
	}
	// roviny, lec na jednotlivch hranch facu

	return true;
}