/*
                                +---------------------------------+
                                |                                 |
                                |       ***   Load3DS   ***       |
                                |                                 |
                                |  Copyright   -tHE SWINe- 2002  |
                                |                                 |
                                |            Load3DS.h            |
                                |                                 |
                                +---------------------------------+
*/

#ifndef __3DS_INCLUDED
#define __3DS_INCLUDED

struct Vertex {
    Vector3f v_pos;
	Vector2f v_tex;
    //
    Vector3f v_norm;
};

struct Material {
    int n_id;
    char *name;
	//
	int n_ogl_id;
};

struct Light {
    int n_id;
    char *name;
    //
    int b_spot;
    int b_off;
    int b_cast_shadows;
    //
    Vector3f v_position;
    Vector3f v_target;
    //
    float f_hotspot;
    float f_faloff;
    //
    float f_inner_range;
    float f_outer_range;
    float f_multiplier;
    //
    Vector3f v_color;
    int r, g, b;
};

struct Face {
    int n_id, n_used;
    int n_material;
    //
    Vertex *vertex[3];
    //
    Planef p_norm;
	//
	int b_Contain_Vertex(Vector3f &v_point);
	//
	Vector3f v_center;
	float f_radius;
};

struct Object {
    int n_id;
    char *name;
    //
    int n_vertex_num;
    int n_face_num;
    //
    Face *face;
    Vertex *vertex;
    //
    Matrix4f m_object;
    //
    int n_parent;
    int n_pos_num;
    int n_rot_num;
    //
    float *pt, *px, *py, *pz;
    float *qt, *qw, *qx, *qy, *qz;
};

struct Camera {
    int n_id;
    char *name;
    //
    Matrix4f m_camera;
    //
    float f_pos_x, f_pos_y, f_pos_z;
    float f_tar_x, f_tar_y, f_tar_z;
    float f_bank_angle;
    float f_focus; 
    //
    int n_target_num;
    int n_position_num;
    float *tt, *tx, *ty, *tz;
    float *pt, *px, *py, *pz;
};

struct World {
	int Load(const char *file_name);
	//
    int n_start_frame;
    int n_end_frame;
    //
    int n_object_num;
    int n_camera_num;
    int n_light_num;
    int n_material_num;
    //
    Object *object;
    Camera *camera;
    Light *light;
    Material *material;
};

#endif // __3DS_INCLUDED
