/*
								+--------------------------------+
								|                                |
								| ***   3DS to header util   *** |
								|                                |
								|  Copyright  -tHE SWINe- 2009  |
								|                                |
								|            Main.cpp            |
								|                                |
								+--------------------------------+
*/

#include <algorithm>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "Vector.h"
#include "Load_3ds.h"
//#define __TEST_GENERATED_FILE__
#ifdef __TEST_GENERATED_FILE__
#include <windows.h>
#include <gl/gl.h>
#include <gl/glext.h>
#include "E:/3dsMax8/meshes/test.h"
#endif //__TEST_GENERATED_FILE__

#if defined(_MSC_VER) && !defined(_MWERKS)
#define for if(0) {} else for
#endif
// msvc 'for' scoping hack

static void DumpTable(const float *p_table, int n_size, int n_align = 100)
{
	int n_line_length = 0;
	for(int i = 0; i < n_size; ++ i) {
		const char *p_s_spacing = " ";
		int n_spacing_length = 1;
		if(!n_line_length) {
			p_s_spacing = "\t";
			n_spacing_length = 4;
		}
		// find spacing

		char p_s_value[16];
		int n_value_length = sprintf(p_s_value, "%ff", float(p_table[i]));
		// create string with value

		const char *p_s_separator = ",";
		int n_separator_length = 1;
		if(i == n_size - 1) {
			p_s_separator = "};\n";
			n_separator_length = 2;
		}
		// find separator

		if(n_line_length + n_value_length + n_separator_length +
		   n_spacing_length >= n_align) {
			p_s_spacing = "\n\t";
			n_spacing_length = 4;
			n_line_length = 0;
		}
		// check line length boundary

		printf("%s%s%s", p_s_spacing, p_s_value, p_s_separator);
		n_line_length += n_value_length + n_separator_length + n_spacing_length;
		// print
	}
}

static void DumpTable(const int *p_table, int n_size, int n_align = 100)
{
	int n_line_length = 0;
	for(int i = 0; i < n_size; ++ i) {
		const char *p_s_spacing = " ";
		int n_spacing_length = 1;
		if(!n_line_length) {
			p_s_spacing = "\t";
			n_spacing_length = 4;
		}
		// find spacing

		char p_s_value[16];
		int n_value_length = sprintf(p_s_value, "%d", int(p_table[i]));
		// create string with value

		const char *p_s_separator = ",";
		int n_separator_length = 1;
		if(i == n_size - 1) {
			p_s_separator = "};\n";
			n_separator_length = 2;
		}
		// find separator

		if(n_line_length + n_value_length + n_separator_length +
		   n_spacing_length >= n_align) {
			p_s_spacing = "\n\t";
			n_spacing_length = 4;
			n_line_length = 0;
		}
		// check line length boundary

		printf("%s%s%s", p_s_spacing, p_s_value, p_s_separator);
		n_line_length += n_value_length + n_separator_length + n_spacing_length;
		// print
	}
}

static bool b_MaterialIdGreater(Face a, Face b)
{
	return a.n_material < b.n_material;
}

int n_MatRange_Num(const World &t_world, int n_object)
{
	int i = n_object;
	int n_mat_range_num = 1;
	for(int j = 1; j < t_world.object[i].n_face_num; ++ j) {
		if(t_world.object[i].face[j - 1].n_material !=
		   t_world.object[i].face[j].n_material)
			++ n_mat_range_num;
	}
	// count number of material ranges

	return n_mat_range_num;
}

int main(int n_arg_num, const char **p_arg_list)
{
	if(n_arg_num != 2) {
		fprintf(stderr, "error: run with single parameter giving path of your .3ds file\n");
		return -1;
	}

	printf("=== Loading \'%s\' ===\n", p_arg_list[1]);
	World t_world;
	if(!t_world.Load(p_arg_list[1])) {
		fprintf(stderr, "error: unable to load .3ds file\n");
		return -1;
	}

	for(int i = 0; i < t_world.n_object_num; ++ i) {
		std::stable_sort(t_world.object[i].face, t_world.object[i].face +
			t_world.object[i].n_face_num, b_MaterialIdGreater);
	}
	// sort faces by materials

	printf("\t%d lights\n", t_world.n_light_num);
	printf("\t%d materials\n", t_world.n_material_num);
	printf("\t%d objects\n", t_world.n_object_num);
	printf("\t%d cameras\n", t_world.n_camera_num);

	char p_s_buffer[1024];
	if(strlen(p_arg_list[1]) > 1023) {
		strncpy(p_s_buffer, p_arg_list[1], 1023 - 2);
		strcpy(p_s_buffer + 1023 - 2, ".h");
		// too long filename
	} else {
		strcpy(p_s_buffer, p_arg_list[1]);
		if(strrchr(p_s_buffer, '.'))
			strcpy(((char*)strrchr(p_s_buffer, '.')), ".h");
		else
			strcat(p_s_buffer, ".h");
		// change extension to .h
	}
	// create outfile

	printf("\n=== Exporting objects geometry to \'%s\' ===\n", p_s_buffer);
	FILE *p_fw;
	if(!(p_fw = fopen(p_s_buffer, "w"))) {
		fprintf(stderr, "error: unable to write to file\n");
		return -1;
	}

	fprintf(p_fw, "/*\n *\t\t\t\t=== static 3DS export by -tHE SWINe- ===\n */\n\n");

	if(strrchr(p_s_buffer, '\\'))
		strcpy(p_s_buffer, strrchr(p_s_buffer, '\\') + 1);
	if(strrchr(p_s_buffer, '/'))
		strcpy(p_s_buffer, strrchr(p_s_buffer, '/') + 1);
	p_s_buffer[strlen(p_s_buffer) - 2] = 0; // remove '.h'
	for(int i = 0, n = strlen(p_s_buffer); i < n; ++ i) {
		if(!isdigit(p_s_buffer[i]) && !isalpha(p_s_buffer[i]) && p_s_buffer[i] != '_')
			p_s_buffer[i] = '_';
	}
	// create valid c++ ident (except it may begin with ident)

	fprintf(p_fw,
		"class C%s_3ds {\n"
		"\tstruct TLight {\n"
		"\t\tconst char *p_s_name;\n"
		"\t\tbool b_spot, b_off, b_cast_shadows;\n"
		"\t\tfloat p_position[4]; // w is always 1\n"
		"\t\tfloat p_target[4]; // w is always 1\n"
		"\t\tfloat f_hotspot, f_fallof;\n"
		"\t\tfloat f_inner_range, f_outer_range, f_multiplier;\n"
		"\t\tfloat p_color[4]; // a is always 1\n"
		"\t};\n\n"
		"\tstruct TMaterial {\n"
		"\t\tconst char *p_s_name;\n"
		"\t\tint n_id;\n"
		"\t};\n\n"
		"\tstruct TObject {\n"
		"\t\tconst char *p_s_name;\n"
		"\t\tint n_id, n_parent;\n"
		"\t\tfloat p_transform[16];\n"
		"\t\tint n_vertex_num, n_face_num;\n"
		"\t\tconst float *p_vertex; // GL_T2F_N3F_V3F\n"
		"\t\tconst unsigned int *p_face;\n"
		"\t\tint n_mat_range_num;\n"
		"\t\tconst struct TMatRange {\n"
		"\t\t\tint n_material;\n"
		"\t\t\tint n_first, n_count; // in vertices\n"
		"\t\t} *p_mat_ranges;\n"
		"\n"
		"\t\tvoid Draw() const\n"
		"\t\t{\n"
		"\t\t\tglInterleavedArrays(GL_T2F_N3F_V3F, 8 * sizeof(float), p_vertex);\n"
		"\t\t\t// supply vertex data\n"
		"\n"
		"\t\t\tglDrawElements(GL_TRIANGLES, 3 * n_face_num, GL_UNSIGNED_INT, p_face);\n"
		"\t\t\t// draw object unsing triangles\n"
		"\n"
		"\t\t\tglDisableClientState(GL_TEXTURE_COORD_ARRAY);\n"
		"\t\t\tglDisableClientState(GL_NORMAL_ARRAY);\n"
		"\t\t\tglDisableClientState(GL_VERTEX_ARRAY);\n"
		"\t\t\t// finished using vertex data\n"
		"\t\t}\n"
		"\t};\n\n"
		"\tstruct TCamera {\n"
		"\t\tconst char *p_s_name;\n"
		"\t\tint n_id;\n"
		"\t\tfloat f_pos_x, f_pos_y, f_pos_z;\n"
		"\t\tfloat f_tar_x, f_tar_y, f_tar_z;\n"
		"\t\tfloat f_bank_angle;\n"
		"\t\tfloat f_focus;\n"
		"\t\tint n_position_num;\n"
		"\t\tint n_target_num;\n"
		"\t\tconst float *tt, *tx, *ty, *tz;\n"
		"\t\tconst float *pt, *px, *py, *pz;\n"
		"\t};\n\n", p_s_buffer);
	// print structures definitions

	for(int i = 0; i < t_world.n_object_num; ++ i) {
		fprintf(p_fw, "\tstatic const float p_object_%d_vertices[8 * %d];\n"
			"\tstatic const unsigned int p_object_%d_indices[3 * %d];\n"
			"\tstatic const TObject::TMatRange p_object_%d_materials[%d];\n",
			i, t_world.object[i].n_vertex_num, i, t_world.object[i].n_face_num,
			i, n_MatRange_Num(t_world, i));
	}
	// print object static arrays definitions

	for(int i = 0; i < t_world.n_camera_num; ++ i) {
		if(t_world.camera[i].n_position_num) {
			fprintf(p_fw, "\tstatic const float p_camera_%d_pos_track[4 * %d];\n",
				i, t_world.camera[i].n_position_num);
		}
		if(t_world.camera[i].n_target_num) {
			fprintf(p_fw, "\tstatic const float p_camera_%d_tar_track[4 * %d];\n",
				i, t_world.camera[i].n_target_num);
		}
	}
	// print camera track static arrays definitions

	if(t_world.n_object_num) {
		fprintf(p_fw,
			"\tstatic const TObject m_p_object[%d];\n", t_world.n_object_num);
	}
	if(t_world.n_light_num) {
		fprintf(p_fw,
			"\tstatic const TLight m_p_light[%d];\n", t_world.n_light_num);
	}
	if(t_world.n_material_num) {
		fprintf(p_fw,
			"\tstatic const TMaterial m_p_material[%d];\n", t_world.n_material_num);
	}
	if(t_world.n_camera_num) {
		fprintf(p_fw,
			"\tstatic const TCamera m_p_camera[%d];\n", t_world.n_camera_num);
	}
	// arrays only if not empty

	fprintf(p_fw,
		"\npublic:\n"
		"\tstatic int n_Object_Num()\n"
		"\t{\n"
		"\t\treturn %d;\n"
		"\t}\n"
		"\n"
		"\tstatic int n_Light_Num()\n"
		"\t{\n"
		"\t\treturn %d;\n"
		"\t}\n"
		"\n"
		"\tstatic int n_Material_Num()\n"
		"\t{\n"
		"\t\treturn %d;\n"
		"\t}\n"
		"\n"
		"\tstatic int n_Camera_Num()\n"
		"\t{\n"
		"\t\treturn %d;\n"
		"\t}\n"
		"\n", t_world.n_object_num, t_world.n_light_num,
		t_world.n_material_num, t_world.n_camera_num);
	// item counts

	fprintf(p_fw,
		"\tstatic const TObject *p_Object()\n"
		"\t{\n"
		"\t\treturn %s;\n"
		"\t}\n"
		"\n", (t_world.n_object_num)? "m_p_object" : "0");
	fprintf(p_fw,
		"\tstatic const TLight *p_Light()\n"
		"\t{\n"
		"\t\treturn %s;\n"
		"\t}\n"
		"\n", (t_world.n_light_num)? "m_p_light" : "0");
	fprintf(p_fw,
		"\tstatic const TMaterial *p_Material()\n"
		"\t{\n"
		"\t\treturn %s;\n"
		"\t}\n"
		"\n", (t_world.n_material_num)? "m_p_material" : "0");
	fprintf(p_fw,
		"\tstatic const TCamera *p_Camera()\n"
		"\t{\n"
		"\t\treturn %s;\n"
		"\t}\n"
		"\n", (t_world.n_camera_num)? "m_p_camera" : "0");
	fprintf(p_fw, "\tstatic void Draw()\n"
		"\t{\n"
		"\t\tfor(int i = 0; i < %d; ++ i)\n"
		"\t\t\tp_Object()[i].Draw();\n"
		"\t}\n"
		"};\n\n", t_world.n_object_num);
	// item arrays

	// print world class

	if(t_world.n_material_num) {
		fprintf(p_fw, "const C%s_3ds::TMaterial C%s_3ds::m_p_material[%d] = {\n",
			p_s_buffer, p_s_buffer, t_world.n_material_num);
		for(int i = 0; i < t_world.n_material_num; ++ i) {
			fprintf(p_fw, "\t{\"%s\", %d}%s\n",
				t_world.material[i].name, t_world.material[i].n_id,
				(i + 1 == t_world.n_material_num)? "" : ",");
		}
		fprintf(p_fw, "};\n\n");
		// print material list
	}
	// print material array

	if(t_world.n_light_num) {
		fprintf(p_fw, "const C%s_3ds::TLight C%s_3ds::m_p_light[%d] = {\n",
			p_s_buffer, p_s_buffer, t_world.n_light_num);
		for(int i = 0; i < t_world.n_light_num; ++ i) {
			fprintf(p_fw, "\t{\"%s\", %s, %s, %s,\n"
				"\t{%ff, %ff, %ff, %ff},\n"
				"\t{%ff, %ff, %ff, %ff},\n"
				"\t%ff, %ff, %ff, %ff, %ff,\n"
				"\t{%ff, %ff, %ff, %ff}}%s\n",
				t_world.light[i].name,
				(t_world.light[i].b_spot)? "true" : "false",
				(t_world.light[i].b_off)? "true" : "false",
				(t_world.light[i].b_cast_shadows)? "true" : "false",
				t_world.light[i].v_position.x,
				t_world.light[i].v_position.y,
				t_world.light[i].v_position.z, 1.0f,
				t_world.light[i].v_target.x,
				t_world.light[i].v_target.y,
				t_world.light[i].v_target.z, 1.0f,
				t_world.light[i].f_hotspot,
				t_world.light[i].f_faloff,
				t_world.light[i].f_inner_range,
				t_world.light[i].f_outer_range,
				t_world.light[i].f_multiplier,
				t_world.light[i].v_color.x,
				t_world.light[i].v_color.y,
				t_world.light[i].v_color.z, 1.0f,
				(i + 1 == t_world.n_light_num)? "" : ",");
		}
		fprintf(p_fw, "};\n\n");
		// print light list
	}
	// print light array

	if(t_world.n_camera_num) {
		for(int i = 0; i < t_world.n_camera_num; ++ i) {
			if(t_world.camera[i].n_position_num) {
				fprintf(p_fw, "const float C%s_3ds::p_camera_%d_pos_track[4 * %d] = {\n\t",
					p_s_buffer, i, t_world.camera[i].n_position_num);
				for(int j = 0; j < t_world.camera[i].n_position_num; ++ j) {
					fprintf(p_fw, "%ff,%s", t_world.camera[i].pt[j],
						(j + 1 == t_world.camera[i].n_position_num)? "\n\t" : ((j % 8 == 7)? "\n\t" : " "));
				}
				for(int j = 0; j < t_world.camera[i].n_position_num; ++ j) {
					fprintf(p_fw, "%ff,%s", t_world.camera[i].px[j],
						(j + 1 == t_world.camera[i].n_position_num)? "\n\t" : ((j % 8 == 7)? "\n\t" : " "));
				}
				for(int j = 0; j < t_world.camera[i].n_position_num; ++ j) {
					fprintf(p_fw, "%ff,%s", t_world.camera[i].py[j],
						(j + 1 == t_world.camera[i].n_position_num)? "\n\t" : ((j % 8 == 7)? "\n\t" : " "));
				}
				for(int j = 0; j < t_world.camera[i].n_position_num; ++ j) {
					fprintf(p_fw, "%ff,%s", t_world.camera[i].pz[j],
						(j + 1 == t_world.camera[i].n_position_num)? "\n" : ((j % 8 == 7)? "\n\t" : " "));
				}
				fprintf(p_fw, "};\n\n");
			}
			if(t_world.camera[i].n_target_num) {
				fprintf(p_fw, "const float C%s_3ds::p_camera_%d_tar_track[4 * %d] = {\n\t",
					p_s_buffer, i, t_world.camera[i].n_target_num);
				for(int j = 0; j < t_world.camera[i].n_target_num; ++ j) {
					fprintf(p_fw, "%ff,%s", t_world.camera[i].tt[j],
						(j + 1 == t_world.camera[i].n_target_num)? "\n\t" : ((j % 8 == 7)? "\n\t" : " "));
				}
				for(int j = 0; j < t_world.camera[i].n_target_num; ++ j) {
					fprintf(p_fw, "%ff,%s", t_world.camera[i].tx[j],
						(j + 1 == t_world.camera[i].n_target_num)? "\n\t" : ((j % 8 == 7)? "\n\t" : " "));
				}
				for(int j = 0; j < t_world.camera[i].n_target_num; ++ j) {
					fprintf(p_fw, "%ff,%s", t_world.camera[i].ty[j],
						(j + 1 == t_world.camera[i].n_target_num)? "\n\t" : ((j % 8 == 7)? "\n\t" : " "));
				}
				for(int j = 0; j < t_world.camera[i].n_target_num; ++ j) {
					fprintf(p_fw, "%ff,%s", t_world.camera[i].tz[j],
						(j + 1 == t_world.camera[i].n_target_num)? "\n" : ((j % 8 == 7)? "\n\t" : " "));
				}
				fprintf(p_fw, "};\n\n");
			}
		}
		// print static camera track data

		fprintf(p_fw, "const C%s_3ds::TCamera C%s_3ds::m_p_camera[%d] = {\n",
			p_s_buffer, p_s_buffer, t_world.n_camera_num);
		for(int i = 0; i < t_world.n_camera_num; ++ i) {
			fprintf(p_fw, "\t{\"%s\", %d, %ff, %ff, %ff, %ff,\n"
				"\t%ff, %ff, %ff, %ff, %d, %d,",
				t_world.camera[i].name, t_world.camera[i].n_id,
				t_world.camera[i].f_pos_x,
				t_world.camera[i].f_pos_y,
				t_world.camera[i].f_pos_z,
				t_world.camera[i].f_tar_x,
				t_world.camera[i].f_tar_y,
				t_world.camera[i].f_tar_z,
				t_world.camera[i].f_bank_angle,
				t_world.camera[i].f_focus,
				t_world.camera[i].n_position_num,
				t_world.camera[i].n_target_num);
			if(t_world.camera[i].n_position_num) {
				fprintf(p_fw, "\n\tp_camera_%d_pos_track,"
					" p_camera_%d_pos_track + %d,\n"
					"\tp_camera_%d_pos_track + %d,"
					" p_camera_%d_pos_track + %d,\n\t",
					i, i, t_world.camera[i].n_position_num,
					i, t_world.camera[i].n_position_num * 2,
					i, t_world.camera[i].n_position_num * 3);
			} else
				fprintf(p_fw, "0, 0, 0, 0,");
			if(t_world.camera[i].n_target_num) {
				fprintf(p_fw, "p_camera_%d_tar_track,"
					" p_camera_%d_tar_track + %d,\n"
					"\tp_camera_%d_tar_track + %d,"
					" p_camera_%d_tar_track + %d",
					i, i, t_world.camera[i].n_target_num,
					i, t_world.camera[i].n_target_num * 2,
					i, t_world.camera[i].n_target_num * 3);
			} else
				fprintf(p_fw, "0, 0, 0, 0");
			fprintf(p_fw, "}%s\n",
				(i + 1 == t_world.n_camera_num)? "" : ",");
		}
		fprintf(p_fw, "};\n\n");
		// print camera list
	}
	// print camera arrays

	if(t_world.n_object_num) {
		for(int i = 0; i < t_world.n_object_num; ++ i) {
			fprintf(p_fw, "/* === static arrays for \'%s\' -> object %d === */\n\n",
				p_arg_list[1], i);

			fprintf(p_fw, "const float C%s_3ds::p_object_%d_vertices[8 * %d] = {\n",
				p_s_buffer, i, t_world.object[i].n_vertex_num);
			for(int j = 0, n = t_world.object[i].n_vertex_num; j < n; ++ j) {
				Vertex v = t_world.object[i].vertex[j];
				fprintf(p_fw, (j + 1 < n)? "\t%ff, %ff, %ff, %ff, %ff, %ff, %ff, %ff,\n" :
					"\t%ff, %ff, %ff, %ff, %ff, %ff, %ff, %ff\n", v.v_tex.x, v.v_tex.y,
					-v.v_norm.x, -v.v_norm.y, -v.v_norm.z, v.v_pos.x, v.v_pos.y, v.v_pos.z);
			}
			fprintf(p_fw, "};\n");
			// print vertex data

			fprintf(p_fw, "const unsigned int C%s_3ds::p_object_%d_indices[3 * %d] = {\n",
				p_s_buffer, i, t_world.object[i].n_face_num);
			char p_s_line[256] = "\t";
			for(int j = 0, n = t_world.object[i].n_face_num; j < n; ++ j) {
				Face f = t_world.object[i].face[j];
				char p_s_face[256];
				sprintf(p_s_face, (j + 1 < n)? "%d, %d, %d, " : "%d, %d, %d ",
					f.vertex[0] - t_world.object[i].vertex,
					f.vertex[1] - t_world.object[i].vertex,
					f.vertex[2] - t_world.object[i].vertex);
				// print face definition

				if(strlen(p_s_line) + strlen(p_s_face) > 80) {
					p_s_line[strlen(p_s_line) - 1] = '\n'; // change the last space for newline
					fprintf(p_fw, "%s", p_s_line);
					// p_s_line[0] = '\t'; // is there all the time
					strcpy(p_s_line + 1, p_s_face);
				} else
					strcat(p_s_line, p_s_face); // just concat, print when long enough
			}
			if(strlen(p_s_line) > 1) {
				p_s_line[strlen(p_s_line) - 1] = '\n'; // change the last space for newline
				fprintf(p_fw, "%s", p_s_line);
				// don't forget last faces!
			}
			fprintf(p_fw, "};\n");
			// print face data

			int n_mat_range_num = n_MatRange_Num(t_world, i);
			fprintf(p_fw, "const C%s_3ds::TObject::TMatRange C%s_3ds::p_object_%d_materials[%d] = {\n",
				p_s_buffer, p_s_buffer, i, n_mat_range_num);
			int n_first = 0;
			for(int j = 0; j < n_mat_range_num; ++ j) {
				int n_last = n_first;
				int n_mat = t_world.object[i].face[n_first].n_material;
				while(n_last < t_world.object[i].n_face_num &&
				   t_world.object[i].face[n_last].n_material == n_mat)
					++ n_last;
				fprintf(p_fw, "\t{%d, %d, %d}%s\n", n_mat, n_first * 3,
					(n_last - n_first) * 3, (j + 1 == n_mat_range_num)? "" : ",");
				n_first = n_last;
			}
			fprintf(p_fw, "};\n");
			// print material data

			fprintf(p_fw, "\n/* === ~static arrays for \'%s\' -> object %d === */\n\n",
				p_arg_list[1], i);
		}
		// static arrays for all objects

		fprintf(p_fw, "const C%s_3ds::TObject C%s_3ds::m_p_object[%d] = {\n",
			p_s_buffer, p_s_buffer, t_world.n_object_num);
		for(int i = 0; i < t_world.n_object_num; ++ i) {
			fprintf(p_fw, "\t{\"%s\", %d, %d, {\n"
				"\t%ff, %ff, %ff, %ff,\n"
				"\t%ff, %ff, %ff, %ff,\n"
				"\t%ff, %ff, %ff, %ff,\n"
				"\t%ff, %ff, %ff, %ff}, %d, %d,\n"
				"\tp_object_%d_vertices, p_object_%d_indices,\n"
				"\t%d, p_object_%d_materials}%s\n",
				t_world.object[i].name, t_world.object[i].n_id,
				int(short(t_world.object[i].n_parent)),
				t_world.object[i].m_object.f[0][0],
				t_world.object[i].m_object.f[0][1],
				t_world.object[i].m_object.f[0][2],
				t_world.object[i].m_object.f[0][3],
				t_world.object[i].m_object.f[1][0],
				t_world.object[i].m_object.f[1][1],
				t_world.object[i].m_object.f[1][2],
				t_world.object[i].m_object.f[1][3],
				t_world.object[i].m_object.f[2][0],
				t_world.object[i].m_object.f[2][1],
				t_world.object[i].m_object.f[2][2],
				t_world.object[i].m_object.f[2][3],
				t_world.object[i].m_object.f[3][0],
				t_world.object[i].m_object.f[3][1],
				t_world.object[i].m_object.f[3][2],
				t_world.object[i].m_object.f[3][3],
				t_world.object[i].n_vertex_num,
				t_world.object[i].n_face_num, i, i,
				n_MatRange_Num(t_world, i), i,
				(i + 1 == t_world.n_object_num)? "" : ",");
		}
		fprintf(p_fw, "};\n");
		// print object list
	}
	// print object arrays

	fprintf(p_fw, "\n/*\n * -end-of-file-\n */\n");

	if(ferror(p_fw))
		fprintf(stderr, "error: not enough space to write file\n");
	else
		printf("\tDone\n");
	fclose(p_fw);

	return 0;
}
