/*
								+----------------------------------+
								|                                  |
								|       ***  Vector math  ***      |
								|                                  |
								|   Copyright  -tHE SWINe- 2003   |
								|                                  |
								|            Vector.cpp            |
								|                                  |
								+----------------------------------+
*/

#include <string.h>
#include <stdio.h>
#include <math.h>
#include "Vector.h"

float epsilon = 1e-2f;


/*
 *        === Planef ===
 */

/*inline*/ Planef::Planef(Vector3f &v_pos, Vector3f &v_norm)
{
	v_normal = v_norm;
	f_dist = - (v_pos / v_normal);
}
	
/*inline*/ Planef::Planef(Vector3f &v_u, Vector3f &v_v, Vector3f &v_pos)
{
	v_normal = v_u * v_v;
	v_normal.Normalize();
	//
	f_dist = - (v_pos / v_normal);
}

/*inline*/ int Planef::Vector_Pos(Vector3f &v_vec)
{
	float f;

	return ((f = Vector_Dist(v_vec)) > epsilon)? Pl_Front : (f < -epsilon)? Pl_Back : Pl_Onplane;
}

/*inline*/ int Planef::Tri_Pos(Vector3f &v_a, Vector3f &v_b, Vector3f &v_c)
{
	int n_front, n_back;

	n_front = 0;
	n_back = 0;
	//
	switch(Vector_Pos(v_a)) {
	case Pl_Front:
		n_front ++;
		break;
	case Pl_Back:
		n_back ++;
		break;
	}
	switch(Vector_Pos(v_b)) {
	case Pl_Front:
		n_front ++;
		break;
	case Pl_Back:
		n_back ++;
		break;
	}
	switch(Vector_Pos(v_c)) {
	case Pl_Front:
		n_front ++;
		break;
	case Pl_Back:
		n_back ++;
		break;
	}
	// spot vertexy ped, za, split
	
	if(n_front == 0) {
		if(n_back == 0)
			return Pl_Onplane;
		return Pl_Back;
	} else if(n_back == 0)
		return Pl_Front;
	// vyhodnot

	return Pl_Split;
}

/*inline*/ float Planef::Vector_Dist(Vector3f &v_vec)
{
	return v_normal.x * v_vec.x + v_normal.y * v_vec.y + v_normal.z * v_vec.z + f_dist;
}

/*inline*/ float Planef::Vector_Dot(Vector3f &v_vec)
{
	return v_normal / v_vec;
}

/*inline*/ Vector3f Planef::Intersect_Ray(Vector3f &v_org, Vector3f &v_dir)
{
	Vector3f v_intersect;
	float t;

	if((t = Vector_Dot(v_dir)) == 0)
		return v_org;

	t = - Vector_Dist(v_org) / t;
	// vzdlenost

	v_intersect = v_org + v_dir * t;

	return v_intersect;
}

/*inline*/ float Planef::Intersect_Ray_t(Vector3f &v_org, Vector3f &v_dir)
{
	float t;

	if((t = Vector_Dot(v_dir)) == 0)
		return 0;

	return - Vector_Dist(v_org) / t;
	// vzdlenost
}

/*inline*/ int Planef::operator == (Planef &p_plane)
{
	if(fabs(fabs(Vector_Dot(p_plane.v_normal)) - 1) < epsilon &&
	   fabs(f_dist - p_plane.f_dist) < epsilon)
		return true;
	return false;
}


/*
 *        === Matrix4f ===
 */

/*inline*/ void Matrix4f::Init()
{
	int i, j;

	for(i = 0; i < 4; i ++) {		
		for(j = 0; j < 4; j ++)
			f[i][j] = (float)(i == j);
	}
}

/*inline*/ void Matrix4f::Translate(Vector3f &v_dir)
{
	Matrix4f tmp;

    tmp.Init();

    tmp.f[3][0] = v_dir.x;
    tmp.f[3][1] = v_dir.y;
    tmp.f[3][2] = v_dir.z;

    _Self_Mult(tmp);
}

/*inline*/ void Matrix4f::RotateX(float f_angle)
{
	Matrix4f tmp;

    tmp.Init();
    tmp.f[2][1] = (float)-sin(f_angle);
    tmp.f[2][2] = (float)cos(f_angle);
    tmp.f[1][1] = (float)cos(f_angle);
    tmp.f[1][2] = (float)sin(f_angle);

    _Self_Mult(tmp);
}

/*inline*/ void Matrix4f::RotateY(float f_angle)
{
	Matrix4f tmp;

    tmp.Init();

    tmp.f[0][0] = (float)cos(f_angle);
    tmp.f[0][2] = (float)-sin(f_angle);
    tmp.f[2][2] = (float)cos(f_angle);
    tmp.f[2][0] = (float)sin(f_angle);

    _Self_Mult(tmp);
}

/*inline*/ void Matrix4f::RotateZ(float f_angle)
{
	Matrix4f tmp;

    tmp.Init();

    tmp.f[0][0] = (float)cos(f_angle);
    tmp.f[0][1] = (float)sin(f_angle);
    tmp.f[1][1] = (float)cos(f_angle);
    tmp.f[1][0] = (float)-sin(f_angle);

    _Self_Mult(tmp);
}

/*inline*/ void Matrix4f::_Self_Mult(Matrix4f &mat)
{
	Matrix4f tmp;
	int i, j;

    for(i = 0; i < 4; i ++) {
        for(j = 0; j < 4; j ++) {
            tmp.f[i][j] = f[0][j] * mat.f[i][0] + f[1][j] * mat.f[i][1] +
						  f[2][j] * mat.f[i][2] + f[3][j] * mat.f[i][3];
		}
    }

	memcpy(f, tmp.f, sizeof(f));
}

/*inline*/ Matrix4f Matrix4f::operator * (Matrix4f &mat)
{
	Matrix4f tmp;
    int i, j;

    for(i = 0; i < 4; i ++) {
        for(j = 0; j < 4; j ++) {
            tmp.f[i][j] = f[0][j] * mat.f[i][0] + f[1][j] * mat.f[i][1] +
						  f[2][j] * mat.f[i][2] + f[3][j] * mat.f[i][3];
		}
    }

    return tmp;
}

/*inline*/ void Matrix4f::operator *= (Matrix4f &mat)
{
	_Self_Mult(mat);
}

/*inline*/ void Matrix4f::operator = (Matrix4f &mat)
{
	memcpy(f, mat.f, sizeof(f));
}

/*inline*/ void Matrix4f::Inverse()
{
	Matrix4f inv;
	float f_det;
    int i, j;

    f_det = f[0][0] * f[1][1] * f[2][2] +
            f[1][0] * f[2][1] * f[0][2] +
            f[0][1] * f[1][2] * f[2][0] -
            f[0][2] * f[1][1] * f[2][0] -
            f[0][1] * f[1][0] * f[2][2] -
            f[0][0] * f[1][2] * f[2][1];
    // spocita determinant podle sarrusova pravidla
    // u matice se spodkem "0 0 0 1" je determinant
    // stejny jako posledni doplnek

    inv.f[0][0] = f[1][1] * f[2][2] - f[1][2] * f[2][1];
    inv.f[1][0] =-f[1][0] * f[2][2] + f[1][2] * f[2][0];
    inv.f[2][0] = f[1][0] * f[2][1] - f[1][1] * f[2][0];
    inv.f[3][0] =-f[1][0] * f[2][1] * f[3][2] -
                  f[1][1] * f[2][2] * f[3][0] -
                  f[2][0] * f[3][1] * f[1][2] +
                  f[1][2] * f[2][1] * f[3][0] +
                  f[1][1] * f[2][0] * f[3][2] +
                  f[2][2] * f[1][0] * f[3][1];
    // takhle moc se to vecko zkrt, dl u to nebudu rozepisovat ...

    inv.f[0][1] =-f[0][1] * f[2][2] + f[0][2] * f[2][1];
    inv.f[1][1] = f[0][0] * f[2][2] - f[0][2] * f[2][0];
    inv.f[2][1] =-f[0][0] * f[2][1] + f[0][1] * f[2][0];
    inv.f[3][1] = f[0][0] * f[2][1] * f[3][2] +
                  f[0][1] * f[2][2] * f[3][0] +
                  f[2][0] * f[3][1] * f[0][2] -
                  f[0][2] * f[2][1] * f[3][0] -
                  f[0][1] * f[2][0] * f[3][2] -
                  f[2][2] * f[0][0] * f[3][1];
    inv.f[0][2] = f[0][1] * f[1][2] - f[0][2] * f[1][1];
    inv.f[1][2] =-f[0][0] * f[1][2] + f[0][2] * f[1][0];
    inv.f[2][2] = f[0][0] * f[1][1] - f[0][1] * f[1][0];
    inv.f[3][2] =-f[0][0] * f[1][1] * f[3][2] -
                  f[0][1] * f[1][2] * f[3][0] -
                  f[1][0] * f[3][1] * f[0][2] +
                  f[0][2] * f[1][1] * f[3][0] +
                  f[0][1] * f[1][0] * f[3][2] +
                  f[1][2] * f[0][0] * f[3][1];

    for(j = 0; j < 4; j ++)
        for(i = 0; i < 3; i ++)
            inv.f[j][i] /= f_det;
    // vecko vydl determinantem (krom poslednho sloupce)

    inv.f[0][3] = 0;
    inv.f[1][3] = 0;
    inv.f[2][3] = 0;
    inv.f[3][3] = 1;
    // pi matici se spodnm dkem "0 0 0 1" vyjde vdycky posledn sloupec takhle.

	memcpy(f, inv.f, sizeof(f));
}

/*inline*/ void Matrix4f::Scale(float f_scale)
{
	f[0][0] *= f_scale;
	f[0][1] *= f_scale;
	f[0][2] *= f_scale;
	//
	f[1][0] *= f_scale;
	f[1][1] *= f_scale;
	f[1][2] *= f_scale;
	//
	f[2][0] *= f_scale;
	f[2][1] *= f_scale;
	f[2][2] *= f_scale;
}

/*inline*/ Vector3f Matrix4f::Transform_Pos(Vector3f &v_vec)
{
	return Vector3f(v_vec.x * f[0][0] + v_vec.y * f[1][0] + v_vec.z * f[2][0] + f[3][0],
				    v_vec.x * f[0][1] + v_vec.y * f[1][1] + v_vec.z * f[2][1] + f[3][1],
				    v_vec.x * f[0][2] + v_vec.y * f[1][2] + v_vec.z * f[2][2] + f[3][2]);
}

/*inline*/ Vector3f Matrix4f::Transform_Dir(Vector3f &v_vec)
{
	return Vector3f(v_vec.x * f[0][0] + v_vec.y * f[1][0] + v_vec.z * f[2][0],
				    v_vec.x * f[0][1] + v_vec.y * f[1][1] + v_vec.z * f[2][1],
				    v_vec.x * f[0][2] + v_vec.y * f[1][2] + v_vec.z * f[2][2]);
}




/*
 *        === Polygon ===
 */

CPolygon::~CPolygon()
{
}

void CPolygon::Free()
{
	if(m_p_vertex != NULL && m_n_vertex_num) {
		delete m_p_vertex;
		m_p_vertex = NULL;
	}
	m_n_vertex_num = 0;
	m_n_vertex_used = 0;
	// uvoln pole vertex
}

int CPolygon::GrowBuffer(int n_grow_num)
{
	Vector3f *p_vertex;

	if(n_grow_num <= 0)
		n_grow_num = 1;
	if(m_p_vertex == NULL) {
		if((m_p_vertex = new Vector3f[n_grow_num]) == NULL)
			return false;
		m_n_vertex_num = n_grow_num;
	} else {
		if((p_vertex = new Vector3f[m_n_vertex_num + n_grow_num]) == NULL)
			return false;
		memcpy(p_vertex, m_p_vertex, m_n_vertex_num * sizeof(Vector3f));
		delete m_p_vertex;
		m_p_vertex = p_vertex;
		//
		m_n_vertex_num += n_grow_num;
	}
	// naalokuje nebo zvt buffer pro vrcholy

	return true;
}

int CPolygon::Add_Vertex(Vector3f v_vertex)
{
	if(m_n_vertex_used >= m_n_vertex_num) {
		if(!GrowBuffer(1))
			return false;
	}
	m_p_vertex[m_n_vertex_used ++] = v_vertex;
	// podle poteby zvt pole a pid vrcholy

	return true;
}

int CPolygon::Add_Vertex(Vector3f *v_vertex, int n_num)
{
	int i;

	for(i = 0; i < n_num; i ++) {
		if(m_n_vertex_used >= m_n_vertex_num) {
			if(!GrowBuffer(n_num - i))
				return false;
		}
		//
		m_p_vertex[m_n_vertex_used ++] = v_vertex[i];
	}
	// podle poteby zvt pole a pid vrcholy

	return true;
}

Vector3f *CPolygon::p_Vertex(int n_index)
{
	if(n_index >= 0 && n_index < m_n_vertex_used)
		return &m_p_vertex[n_index];
	// vrt vrchol podle indexu
	return NULL;
}

int CPolygon::Calc_Normal()
{
	Vector3f v_u, v_v;

	if(m_n_vertex_used < 3)
		return false;

	v_u = m_p_vertex[0] - m_p_vertex[2];
	v_v = m_p_vertex[1] - m_p_vertex[2];
	//
	if(v_u.Len2() == 0 || v_v.Len2() == 0)
		return false;
	// zkontroluje, zda m rovina smysl

	m_p_normal = Planef(v_u, v_v, m_p_vertex[2]);
	// spot rovinu

	return true;
}

int CPolygon::Plane_Pos(Planef &p_plane)
{
	int i, n_front, n_back;

	for(i = 0, n_front = 0, n_back = 0; i < m_n_vertex_used; i ++) {
		switch(p_plane.Vector_Pos(m_p_vertex[i])) {
		case Pl_Front:
			n_front ++;
			break;
		case Pl_Back:
			n_back ++;
			break;
		}
	}
	//
	if(n_back == 0) {
		if(n_front == 0)
			return Pl_Onplane;
		return Pl_Front;
	} else {
		if(n_front == 0)
			return Pl_Back;
		return Pl_Split;
	}
	// vyhodnot pozici polygonu proti rovin
}

int CPolygon::Cut(Planef &p_plane, int n_desired_half)
{
	int n_side, n_prev_side;
	int n_prev_vertex_num;
	Vector3f *p_vertex;
	int n_vertex_num;
	int i, n_prev;

	if(m_n_vertex_used < 3)
		return false;
	// pokud je to polygon

	if((p_vertex = new Vector3f[n_prev_vertex_num = m_n_vertex_used + 1]) == NULL)
		return false;
	n_vertex_num = 0;
	// pole pro doasn uloen vsledku

	n_prev = m_n_vertex_used - 1;
	n_prev_side = p_plane.Vector_Pos(m_p_vertex[n_prev]);
	for(i = 0; i < m_n_vertex_used; n_prev = i, n_prev_side = n_side, i ++) {
		n_side = p_plane.Vector_Pos(m_p_vertex[i]);
		//
		if(n_prev_side != n_desired_half && n_prev_side != Pl_Onplane) {
			if(n_side == n_desired_half) {
				p_vertex[n_vertex_num ++] = p_plane.Intersect_Ray
					(m_p_vertex[i], m_p_vertex[n_prev] - m_p_vertex[i]);
				p_vertex[n_vertex_num ++] = m_p_vertex[i];
			} else if(n_side == Pl_Onplane)
				p_vertex[n_vertex_num ++] = m_p_vertex[i];
		} else {
			if(n_side == n_desired_half || n_side == Pl_Onplane)
				p_vertex[n_vertex_num ++] = m_p_vertex[i];
			else if(n_prev_side != Pl_Onplane) {
				p_vertex[n_vertex_num ++] = p_plane.Intersect_Ray
					(m_p_vertex[i], m_p_vertex[n_prev] - m_p_vertex[i]);
			}
		}
	}
	// rozdl polygon do doasnho pole

	if(n_prev_vertex_num < n_vertex_num)
		return false;
	// singularita

	Free();
	m_p_vertex = p_vertex;
	m_n_vertex_num = n_prev_vertex_num;
	m_n_vertex_used = n_vertex_num;
	// odstran star vrcholy a nahrad je novmi

	return true;
}

int CPolygon::Split(Planef &p_plane, CPolygon &p_new_poly, int n_desired_half)
{
	if(m_n_vertex_used < 3)
		return false;
	// pokud je to polygon

	p_new_poly = *this;
	return p_new_poly.Cut(p_plane, n_desired_half);
	// zkopruje a dl
}

int CPolygon::b_Contain_Point(Vector3f v_point)
{
	Vector3f v_vec, v_prev_vec;
	float f_angle_sum;
	int i;

	v_prev_vec = m_p_vertex[m_n_vertex_used - 1] - v_point;
	v_prev_vec.Normalize();
	//
	for(i = 0, f_angle_sum = 0; i < m_n_vertex_used; i ++, v_prev_vec = v_vec) {
		v_vec = m_p_vertex[m_n_vertex_used - 1] - v_point;
		v_vec.Normalize();
		//
		f_angle_sum += (float)fabs(acos(v_vec / v_prev_vec));
	}
	// spot sumu vech hl v polygonu

	if(f_angle_sum - 2 * Pi < epsilon)
		return true;
	// pokud je suma 360, le bod uvnit

	return false;
}

void CPolygon::Write(FILE *p_file)
{
	fwrite(&m_n_vertex_used, sizeof(int), 1, p_file);
	fwrite(m_p_vertex, sizeof(Vector3f), m_n_vertex_used, p_file);
}

int CPolygon::Read(FILE *p_file)
{
	fread(&m_n_vertex_used, sizeof(int), 1, p_file);
	if(!GrowBuffer(m_n_vertex_used))
		return false;
	fread(m_p_vertex, sizeof(Vector3f), m_n_vertex_used, p_file);

	return true;
}

void CPolygon::operator = (CPolygon &poly)
{
	Free();
	//
	m_n_vertex_used = poly.m_n_vertex_used;
	m_n_vertex_num = poly.m_n_vertex_num;
	//
	if((m_p_vertex = new Vector3f[m_n_vertex_num]) == NULL) {
		m_n_vertex_used = 0;
		m_n_vertex_num = 0;
	}
	memcpy(m_p_vertex, poly.m_p_vertex, m_n_vertex_used * sizeof(Vector3f));
	//
	m_p_normal = poly.m_p_normal;
	// zkopruje data polygonu
}

void CPolygon::Reverb()
{
	Vector3f v_temp;
	int i;

	for(i = 0; i < m_n_vertex_used / 2; i ++) {
		v_temp = m_p_vertex[i];
		m_p_vertex[i] = m_p_vertex[m_n_vertex_used - i - 1];
		m_p_vertex[m_n_vertex_used - i - 1] = v_temp;
	}
	// prohod vrcholy
}

int CPolygon::b_Overlap(CPolygon &poly)
{
	CPolygon p_tmp_poly;
	Planef p_edge;
	int n_prev;
	int i;

	p_tmp_poly = poly;

	n_prev = m_n_vertex_used - 1;
	for(i = 0; i < m_n_vertex_used; n_prev = i ++) {
		p_edge = Planef(m_p_normal.v_normal,
			m_p_vertex[i] - m_p_vertex[n_prev], m_p_vertex[i]);
		//
		p_tmp_poly.Cut(p_edge, Pl_Front);
		if(p_tmp_poly.n_Vertex_Num() < 3)
			return false;
	}
	// na hranch this vytvo roviny, pomoc kterch kontroluje, jestli poly
	// pekrv this (alespo st); pedpokld e oba le ve stejn rovin

	return true;
}

int CPolygon::b_IsTiny()
{
	Vector3f v_dist;
	int i, n_prev;
	int n_valid;

	if(m_n_vertex_used < 3)
		return true;

	n_prev = m_n_vertex_used - 1;
	for(i = 0, n_valid = 0; i < m_n_vertex_used; n_prev = i ++) {
		if((m_p_vertex[i] - m_p_vertex[n_prev]).Len2() > epsilon)
			if(++ n_valid == 3)
				return false;
	}
	// zjist, jestli polygon obsahuje njak hrany, tak dlouh aby se neztratil

	return true;
}